"""MiTepid - python-based Tool to simulate an epidemilogical model for the spread of COVID-19. """

__version__ = '0.0.1'
__author__ = 'Vahid Samadi Bokharaie <vahid.bokharaie@tuebingen.mpg.de>'
__all__ = [] 




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Mar 20 12:27:48 2020.

@author: vbokharaie
"""



def load_mat(filename, var_name=None):
    """
    Load data from mat files.

    Parameters
    ----------
    filename : pathlib Path
        filename including possibly multiple variable in matlab mat format.
    var_name : str, optional
        variable to be loaded. The default is None.

    Returns
    -------
    my_data : TYPE
        DESCRIPTION.

    """
    from scipy.io import loadmat

    data_struct = loadmat(filename)
    if not var_name:
        var_name = [x for x in data_struct.keys() if not '__' in x][0]
    my_data = data_struct[var_name]
    return my_data


def save_mat(filename, var):
    """
    Save data to mat files.

    Parameters
    ----------
    filename : pathlib Path
        filename.
    var : str
        variable name.

    Returns
    -------
    None.

    """
    from scipy.io import savemat

    folder = filename.parent
    folder.mkdir(parents=True, exist_ok=True)
    my_dict = {'var_name': var}
    savemat(filename, my_dict)

def sol_aggregate(sol_in, pop_country):
    """
    Calculate the aggregate trajectory of the whole population.

    Each age groups is weighted based on its ratio of the whol population.

    Parameters
    ----------
    sol_in : numpy array
        solution to epid model. shape (N_time x N_groups)
    pop_country : list of floats
        list of relative ratio of each age group in the population.

    Returns
    -------
    sol_out : numpy array
        aggregate trajectory. shape (N_time x 1)

    """
    import numpy as np

    sol = sol_in.copy()
    Ng = sol.shape[1]
    for cc in np.arange(Ng):
        sol[:,cc] = sol[:,cc] * pop_country[cc]/100
    sol_out = np.sum(sol, axis=1)
    sol_out = sol_out.reshape(sol_out.size,1)
    return sol_out

def scale_B_opt(B_opt_in, list_scales=None):
    """
    Scale matrix of contact ratios for a given policy.

    Parameters
    ----------
    B_opt_in : numpy 2D array
        matrix of contact rates.
    list_scales : list floats, optional
        how each row of B_opt_in should be scaled.
        The default is None, in which case return B_opt_in.

    Returns
    -------
    B_opt_out : numpy 2D array
        scaled contact rates.
    """
    list_scales = list(list_scales)
    Ng = B_opt_in.shape[0]
    if len(list_scales)==1:
        list_scales = [list_scales[0]] * Ng
    elif not len(list_scales) == Ng:
        return

    B_opt_out = B_opt_in.copy()
    for idx, el in enumerate(list_scales):
        B_opt_out[idx, :] = B_opt_out[idx, :] * list_scales[idx]

    return B_opt_out

def sort_out_t_policy_x0(policy_definition, xternal_inputs, t_end, Ng):
    """
    Take the list of policies and external inputs, turn them into list of times policies.

    Parameters
    ----------
    policy_definition : dict
        dictionary of time: polciy.
    xternal_inputs : TYPE
        DESCRIPTION.
    t_end : float
        end time, in days.
    Ng : int
        Number of groups defined in the model.

    Returns
    -------
    list_t1 : list of float
        list of begin times for each policy.
    list_t2 : list of float
        list of end times for each policy.
    list_policies : list str
        list of policy names.
    list_x0 : list float
        list initial conditions for each policy, in case there is external input.
    list_t_switch : list float
        original switching times in policy definition.
        This would be different than list_t1, is there is external input without policy change.
    list_all_policies : list str
        list of all policies. Would be different than list_policies if there is
        external input without policy change.

    """
    import numpy as np

    list_t_switch = list(policy_definition.keys())
    list_all_policies = list(policy_definition.values())
    list_x0_xtrnl_dummy = [list(np.zeros(Ng))]*len(list_all_policies)

    list_t_xtrnl = list(xternal_inputs.keys())
    list_x0_xtrnl = list(xternal_inputs.values())
    list_all_policies_dummy = [None]*len(list_x0_xtrnl)

    for idx, t in enumerate(list_t_switch):
        try:
            list_idx = [idx2 for idx2, x in enumerate(list_t_xtrnl) if x == t]
            idx2 = list_idx[0]
        except IndexError:
            continue
        list_x0_xtrnl_dummy[idx] = list_x0_xtrnl[idx2]
        list_t_xtrnl.pop(idx2)
        list_all_policies_dummy.pop(idx2)
        list_x0_xtrnl.pop(idx2)

    list_t = list_t_switch + list_t_xtrnl
    list_policies = list_all_policies + list_all_policies_dummy
    list_x0 = list_x0_xtrnl_dummy + list_x0_xtrnl


    ind_sorted = np.argsort(list_t)
    list_t1 = [list_t[x] for x in ind_sorted]

    list_t2 = list_t1.copy()
    list_t2.append(t_end + 1e-10)
    list_t2.pop(0)
    list_policies = [list_policies[x] for x in ind_sorted]
    list_x0 = [list_x0[x] for x in ind_sorted]

    for idx3 in np.arange(len(list_policies)-1)+1:
        if not list_policies[idx3]:
            list_policies[idx3] = list_policies[idx3-1]

    return list_t1, list_t2, list_policies, list_x0, list_t_switch, list_all_policies


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Mar 23 10:07:11 2020.

@author: vbokharaie

You can use this script as a template to run the simulations of an epidemilogicla model as
explained in:
http://people.tuebingen.mpg.de/vbokharaie/pdf_files/Quantifying_COVID19_Containment_Policies.pdf

"""

if __name__ == '__main__':
    from mitepid.func_main import main
    from mitepid.policies import str_policy_info
    from mitepid.plots import bplot

    from pathlib import Path
    import numpy as np

    #%% print policy info text?
    if_print_policies_info = False
    if if_print_policies_info:
        print(str_policy_info())
    subfolder = Path('sample_outputs')
    dir_save_plots_main = Path(Path(__file__).resolve().parent, subfolder)

    ##############################################################################################
    # UNCONTAINED SCENARIO
    ##############################################################################################
    #%% Uncontained in one country
    # See list of defined countries in get_pop_distr in policies module.
    # any country whose age distribution is defeind can be used in here.
    country = 'Iran'
    t_end = 541
    list_t_switch = [0,]
    all_policies = ['Uncontained',]
    policy_definition = dict(zip(list_t_switch, all_policies))
    policy_name = 'Uncontained'
    x00 = 1e-4  # 0.0001 population in Iran = 8500 people
    x0_vec=[x00, x00, x00, x00, x00, x00, x00, x00, x00,]
    dict_uncontained = main(country, policy_name, policy_definition,
                            dir_save_plots_main, t_end=t_end, x0_vec=x0_vec)


    #%% Uncontained scenario, various countries
    t_end = 541
    list_t_switch = [0]
    policy_list = ['Uncontained',
                   ]
    list_countries = ['Germany',
                   'Iran',
                   'Italy',
                   'Spain',
                   'China',
                   'USA',
                   'UK',
                   'France',
                   ]
    dict_I = {}
    dict_R = {}
    for country in list_countries:
        print('**************************************')
        print(country)
        all_policies = ['Uncontained',]
        policy_definition = dict(zip(list_t_switch, all_policies))
        x0_vec=[x00, x00, x00, x00, x00, x00, x00, x00, x00,]
        policy_name = 'Uncontained'
        dict_current = main(country, policy_name, policy_definition,
                            dir_save_plots_main, t_end=t_end, x0_vec=x0_vec)

        dict_I[country] = dict_current['sol_agg_SIR_I']
        dict_R[country] = dict_current['sol_agg_SIR_R']

    for idx, country in enumerate(dict_I.keys()):

        if idx == 0:
            sol_agg_SIR_R_plot = dict_R[country]
            sol_agg_SIR_I_plot = dict_I[country]
        else:
            sol_agg_SIR_R_plot = np.concatenate((sol_agg_SIR_R_plot,
                                                 dict_R[country]), axis=1)
            sol_agg_SIR_I_plot = np.concatenate((sol_agg_SIR_I_plot,
                                                 dict_I[country]), axis=1)

    dir_save_plots_country = Path(dir_save_plots_main)
    dir_save_plots = Path(dir_save_plots_country, '00_OVERALL_Uncontained')
    dir_save_plots.mkdir(exist_ok=True, parents=True)
    filesave_I = Path(dir_save_plots, 'SIR_I_AGG_ALL_countries.png')
    filesave_R = Path(dir_save_plots, 'SIR_R_AGG_ALL_countries.png')

    t = np.arange(0, t_end+.01, step=0.1)
    list_all_policies = ['Uncontained']
    bplot(t, sol_agg_SIR_R_plot, plot_type=1, filesave=filesave_R,
      suptitle='', labels=list_countries, list_vl=list_t_switch,
      list_all_policies=list_all_policies, ylabel='Recovered Ratio', cmap='Set1')
    bplot(t, sol_agg_SIR_I_plot, plot_type=1, filesave=filesave_I,
      suptitle='', labels=list_countries, list_vl=list_t_switch,
      list_all_policies=list_all_policies, ylabel='Infectious Ratio', cmap='Set1')

    ##############################################################################################
    # SUPPRESSION PLANS
    ##############################################################################################
    #%% Ninety days of unconatined scenario in Iran, then trying various policies.
    country = 'Iran'
    t_end = 541
    list_t_switch = [0, 90]
    policy_list = ['Uncontained',
                   'Schools_closed',
                   'Elderly_self_isolate',
                   'Kids_Elderly_self_isolate',
                   'Schools_Offices_closed',
                   'Adults_Elderly_Self_isolate',
                   'Social_Distancing',
                   'Lockdown',
                   ]
    policy_labels = ['UN',
                   'KI',
                   'EL',
                   'KIEL',
                   'KIOF',
                   'ADEL',
                   'SD',
                   'LD',
                   ]
    dict_I = {}
    dict_R = {}
    for policy in policy_list:
        all_policies = ['Uncontained',]
        all_policies.append(policy)
        policy_definition = dict(zip(list_t_switch, all_policies))
        x0_vec=[x00, x00, x00, x00, x00, x00, x00, x00, x00,]
        policy_name = 'Uncontained_then_' + policy
        print('*********************************************')
        print(policy_name)
        dict_current = main(country, policy_name, policy_definition,
                            dir_save_plots_main, t_end=t_end, x0_vec=x0_vec)

        dict_I[policy] = dict_current['sol_agg_SIR_I']
        dict_R[policy] = dict_current['sol_agg_SIR_R']

    for idx, policy in enumerate(dict_I.keys()):

        if idx == 0:
            sol_agg_SIR_R_plot = dict_R[policy]
            sol_agg_SIR_I_plot = dict_I[policy]
        else:
            sol_agg_SIR_R_plot = np.concatenate((sol_agg_SIR_R_plot,
                                                 dict_R[policy]), axis=1)
            sol_agg_SIR_I_plot = np.concatenate((sol_agg_SIR_I_plot,
                                                 dict_I[policy]), axis=1)

    dir_save_plots_country = Path(dir_save_plots_main, country)
    dir_save_plots = Path(dir_save_plots_country, '00_OVERALL')
    dir_save_plots.mkdir(exist_ok=True, parents=True)
    filesave_I = Path(dir_save_plots, 'SIR_I_AGG_ALL_policies_IRAN.png')
    filesave_R = Path(dir_save_plots, 'SIR_R_AGG_ALL_policies_IRAN.png')

    # plot aggregate plots to compare all policies.
    from plots import bplot
    t = np.arange(0, t_end+.01, step=0.1)
    list_all_policies = ['Uncontained']
    list_all_policies.extend(['Policy']*(len(policy_list)-1))
    bplot(t, sol_agg_SIR_R_plot, plot_type=1, filesave=filesave_R,
      suptitle='', labels=policy_labels, list_vl=list_t_switch,
      list_all_policies=list_all_policies, ylabel='Recovered Ratio')
    bplot(t, sol_agg_SIR_I_plot, plot_type=1, filesave=filesave_I,
      suptitle='', labels=policy_labels, list_vl=list_t_switch,
      list_all_policies=list_all_policies, ylabel='Infectious Ratio')

    ##############################################################################################
    # LONGTERM PLANS
    ##############################################################################################
    #%% Switching between Uncontained and policy that enforces R0 = 1.0
    country = 'Iran'
    t_end = 541
    list_t_switch = [0, 90, 120, 150, 180, 210, 240, 270, 300, 330, 360, ]
    all_policies = ['Uncontained',
                    'R0_is_1',
                    'Uncontained',
                    'R0_is_1',
                    'Uncontained',
                    'R0_is_1',
                    'Uncontained',
                    'R0_is_1',
                    'Uncontained',
                    'R0_is_1',
                    'Uncontained',
                   ]
    policy_definition = dict(zip(list_t_switch, all_policies))
    x0_vec=[x00, x00, x00, x00, x00, x00, x00, x00, x00,]
    policy_name = 'Uncontained_then_switching_R0'
    dict_current = main(country, policy_name, policy_definition,
                            dir_save_plots_main, t_end=t_end, x0_vec=x0_vec)


    #%% Switching between Uncontained and Lockdown policy
    country = 'Iran'
    t_end = 541
    list_t_switch = [0, 90, 120, 150, 180, 210, 240, 270, 300, 330, 360,]
    all_policies = ['Uncontained',
                    'Lockdown',
                    'Uncontained',
                    'Lockdown',
                    'Uncontained',
                    'Lockdown',
                    'Uncontained',
                    'Lockdown',
                    'Uncontained',
                    'Lockdown',
                    'Uncontained',
                   ]
    policy_definition = dict(zip(list_t_switch, all_policies))
    x0_vec=[x00, x00, x00, x00, x00, x00, x00, x00, x00,]
    policy_name = 'Uncontained_then_switching_Lockdown'
    dict_current = main(country, policy_name, policy_definition,
                            dir_save_plots_main, t_end=t_end, x0_vec=x0_vec)


    #%% Switching between Uncontained, policy that enforces R0 = 1.0 and Lockdown
    t_end = 541
    list_t_switch = [0, 90, 120, 150, 180, 210, 240, 270, 300, 330,]
    all_policies = ['Uncontained',
                    'Lockdown',
                    'R0_is_1',
                    'Uncontained',
                    'Lockdown',
                    'R0_is_1',
                    'Uncontained',
                    'Lockdown',
                    'R0_is_1',
                    'Uncontained',
                     ]

    policy_definition = dict(zip(list_t_switch, all_policies))
    x0_vec=[x00, x00, x00, x00, x00, x00, x00, x00, x00,]
    policy_name = 'Uncontained_then_switching_Lockdown_R0'
    dict_current = main(country, policy_name, policy_definition,
                            dir_save_plots_main, t_end=t_end, x0_vec=x0_vec)


    ##############################################################################################



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Mar 20 12:59:33 2020.

@author: vbokharaie
"""

def main(country,
         policy_name,
         policy_definition,
         dir_save_plots_main='',
         t_end=541,
         x0_vec = [1e-3],
         xternal_inputs={},
         if_title=False):
    """
    Simulate the epid model.

    Calculate trajectory of an epid model for a country and for a defined policy.

    Parameters
    ----------
    country : str
        Name of the country, for which population distribution is defined in the code.
    policy_name : str
        a given name to oplicy used as subfolder name.
    policy_definition : dict
        a dictionary of {time: basic_policy} format.
    dir_save_plots_main : pathlib Path
        main folder to save plots. Then a subfolder in country/policyname is used to save files.
    xternal_inputs : dict
        dictionary of time/xternal_input. To model new Infectiouss fro moutside the populations
            example: incoming flights, etc.
    t_end : float
        end time of simulation, in days
    x0_vec : list of floats
        a list of size 1, Ng, or 2*Ng to specifiy initial conditions. Ng is number of groups.

    Returns
    -------
    dict_out : dictionary
        a dicitonary of all stratified and aggregated solutions of SIS and SIR models.

    """
    # local imports
    from mitepid.models import SIS, SIR
    from mitepid.plots import bplot
    from mitepid.utils import sol_aggregate, load_mat
    from mitepid.utils import sort_out_t_policy_x0
    from mitepid.policies import get_Bopt, str_policy_info, get_pop_distr

    # external imports
    from scipy.integrate import odeint
    import numpy as np
    from numpy.linalg import inv, eigvals
    from pathlib import Path

    print('***************************************************')
    # assuming data files are where the .py file is:
    dir_source_mat = Path(Path(__file__).resolve().parent)
    file_data_opt_SIR = Path(dir_source_mat, 'Optimised_B', 'SIR_B_opt_normalised.mat')
    file_data_opt_SIS = Path(dir_source_mat, 'Optimised_B', 'SIS_B_opt_normalised.mat')

    dir_save_plots_country = Path(dir_save_plots_main, country)

    B_opt_SIS_orig = get_Bopt(file_data_opt_SIS, country, 'Uncontained', )
    B_opt_SIR_orig = get_Bopt(file_data_opt_SIR, country, 'Uncontained', )

    Ng = B_opt_SIS_orig.shape[0]  # number of age groups

    # making the D matrix, alpha represents recovery rate
    alpha = load_mat(file_data_opt_SIR, 'alpha')
    ALPHA = np.ones((Ng, 1)) * alpha
    D = np.zeros((Ng,Ng))
    np.fill_diagonal(D, ALPHA)

    # age groups, used as legends in plots
    age_groups = ['[0-10]',  '[10-20]', '[20-30]', '[30-40]',
                  '[40-50]', '[50-60]', '[60-70]', '[70-80]', '80+',]

    #
    if policy_name == 'Uncontained':
        print('policy is uncontained')
        if_uncontained = True
    else:
        if_uncontained = False


    # initial conditions, t_f
    if len(x0_vec) == 1:
        x0_SIS = np.ones(Ng)*x0_vec
        x0_SIR = np.concatenate((np.ones(Ng)*x0_vec, np.zeros(Ng)))
    elif len(x0_vec) == Ng:
        x0_SIS = x0_vec
        x0_SIR = np.concatenate((x0_vec, np.zeros(Ng)))
    elif len(x0_vec) == 2*Ng:
        x0_SIS = x0_vec[:Ng]
        x0_SIR = x0_vec
    else:
        raise('Something wrong with initial conditions vector!')
    # uncontained solution
    t = np.arange(0, t_end+.01, step=0.1)

    sol_SIS_orig = odeint(SIS, x0_SIS, t, args=(B_opt_SIS_orig, ALPHA))
    sol_SIR_orig = odeint(SIR, x0_SIR, t, args=(B_opt_SIR_orig, ALPHA))
    sol_SIR_I_orig = sol_SIR_orig[:,:Ng]
    sol_SIR_R_orig = sol_SIR_orig[:,Ng:]
    sol_agg_SIS_orig = sol_aggregate(sol_SIS_orig, get_pop_distr(country))
    sol_agg_SIR_I_orig = sol_aggregate(sol_SIR_I_orig, get_pop_distr(country))
    sol_agg_SIR_R_orig = sol_aggregate(sol_SIR_R_orig, get_pop_distr(country))

    list_t1, list_t2, list_policies, list_x0, list_t_switch, list_all_policies = \
        sort_out_t_policy_x0(policy_definition, xternal_inputs, t_end, Ng)

    str_policy = policy_name  # used to make subfolder name
    dir_save_plots = Path(dir_save_plots_country, Path(str_policy))
    dir_save_plots.mkdir(exist_ok = True, parents=True)
    print('Saving in:')
    print(dir_save_plots)
    file_policy = Path(dir_save_plots, 'policy_details.txt')
    with open(file_policy, 'w') as f:
        f.write('\n    time (day) --->  Policy')
        f.write('\n    ------------------------------------------')

    for idx in np.arange(len(list_t1)):
        t_switch1 = list_t1[idx]
        t_switch2 =  list_t2[idx]
        policy = list_policies[idx]
        x0_xtrnal = list_x0[idx]
        with open(file_policy, 'a') as my_tex:
            my_tex.write('\n %10.1f    --->  %s'%(t_switch1, policy))  # remove previous contents

        B_opt_SIS = get_Bopt(file_data_opt_SIS, country, policy)
        B_opt_SIR = get_Bopt(file_data_opt_SIR, country, policy)
        #%% R_0 policy
        rho = np.max(np.abs(eigvals(np.matmul(-inv(D),B_opt_SIR))))  # spectral radius of -inv(D)*B
        print('%s   ---> R_0 = %2.2f'% (policy, rho))

        t_step = np.arange(t_switch1, t_switch2, step=0.1)
        x0_SIS = np.array(x0_SIS) + np.array(x0_xtrnal)
        x0_SIR = np.array(x0_SIR) + np.concatenate((x0_xtrnal, np.zeros(Ng)))
        # solve the ODE
        sol_SIS_step = odeint(SIS, x0_SIS, t_step, args=(B_opt_SIS, ALPHA))
        sol_SIR_step = odeint(SIR, x0_SIR, t_step, args=(B_opt_SIR, ALPHA))

        # update x0
        x0_SIS = sol_SIS_step[-1]
        x0_SIR = sol_SIR_step[-1]

        if idx == 0:
            sol_SIR = sol_SIR_step
            sol_SIS = sol_SIS_step
        else:
            sol_SIR = np.concatenate((sol_SIR, sol_SIR_step))
            sol_SIS = np.concatenate((sol_SIS, sol_SIS_step))

    sol_SIR_I = sol_SIR[:,:Ng]
    sol_SIR_R = sol_SIR[:,Ng:]
    with open(file_policy, 'a') as my_tex:
        my_tex.write('\n \n \n ')
        my_tex.write(str_policy_info())  # just to remove previous contents
    # calculate aggregate solutions
    sol_agg_SIS = sol_aggregate(sol_SIS, get_pop_distr(country))
    sol_agg_SIR_I = sol_aggregate(sol_SIR_I, get_pop_distr(country))
    sol_agg_SIR_R = sol_aggregate(sol_SIR_R, get_pop_distr(country))

    # save various solutions in a sictioanry to return to the main function
    dict_out = {}
    dict_out['sol_agg_SIR_R'] = sol_agg_SIR_R
    dict_out['sol_agg_SIR_I'] = sol_agg_SIR_I
    dict_out['sol_agg_SIS'] = sol_agg_SIS

    dict_out['sol_SIR_R'] = sol_SIR_R
    dict_out['sol_SIR_I'] = sol_SIR_I
    dict_out['sol_SIS'] = sol_SIS
    dict_out['t'] = t

    sol_SIR_I_plot = np.concatenate((sol_SIR_I, sol_SIR_I_orig), axis=1)
    sol_SIR_R_plot = np.concatenate((sol_SIR_R, sol_SIR_R_orig), axis=1)
    sol_SIS_plot = np.concatenate((sol_SIS, sol_SIS_orig), axis=1)

    if not dir_save_plots_main:
        if_plot = False  #  dir_save_plots_main=='' means don't plot
    else:
        plot_type = 1  # can be 1 -> all in one subplots, or 2 -> each age group in one subplot
        if_plot = True
    if if_plot:
        ### SIS
        filesave = Path(dir_save_plots, 'SIS_groups_' + str_policy+'_tf_'+str(int(t_end))+'.png')
        suptitle = country + ' --- SIS'
        bplot(t, sol_SIS_plot, plot_type=1, filesave=filesave,
              labels=age_groups, suptitle=suptitle, list_vl=list_t_switch,
              list_all_policies=list_all_policies, ylabel='Infectious Ratio',
              Ng=9, cmap='viridis')

        ### SIR_I
        suptitle = ''

        filesave = Path(dir_save_plots, 'SIR_I_groups_' + str_policy+'_tf_'+str(int(t_end))+'.png')
        bplot(t, sol_SIR_I_plot, plot_type=1, filesave=filesave,
              labels=age_groups, suptitle=suptitle, list_vl=list_t_switch,
              list_all_policies=list_all_policies, ylabel='Infectious Ratio',
              Ng=9, cmap='viridis')
        filesave = Path(dir_save_plots, 'SIR_I_groups_multi_ax_' + str_policy+'_tf_'+str(int(t_end))+'.png')
        bplot(t, sol_SIR_I_plot, plot_type=2, filesave=filesave,
              labels=age_groups, suptitle=suptitle, list_vl=list_t_switch,
              list_all_policies=list_all_policies, ylabel='Infectious Ratio',
              Ng=9, cmap='Dark2')

        ### SIR_R
        suptitle = ''
        filesave = Path(dir_save_plots, 'SIR_R_groups_' + str_policy+'_tf_'+str(int(t_end))+'.png')
        bplot(t, sol_SIR_R_plot, plot_type=1, filesave=filesave,
              labels=age_groups, suptitle=suptitle, list_vl=list_t_switch,
              list_all_policies=list_all_policies, ylabel='Recoverd Ratio',
              Ng=9, cmap='viridis', policy_name_pos=0.5)

        suptitle = ''
        filesave = Path(dir_save_plots, 'SIR_R_groups_multi_ax_' + str_policy+'_tf_'+str(int(t_end))+'.png')
        bplot(t, sol_SIR_R_plot, plot_type=2, filesave=filesave,
              labels=age_groups, suptitle=suptitle, list_vl=list_t_switch,
              list_all_policies=list_all_policies, ylabel='Recoverd Ratio',
              Ng=9, cmap='Dark2', policy_name_pos=0.5)

        ### Aggregate SIS
        if if_uncontained:
            my_labels_I = ['Uncontained']
            my_labels_R = ['Uncontained']

        else:
            my_labels_I = ['Uncontained',
                           'policy',]
            my_labels_R = ['Uncontained',
                           'policy',]

        filesave = Path(dir_save_plots, 'SIS_AGG_' + str_policy+'_tf_'+str(int(t_end))+'.png')

        str_max1 = "{:2.2f}".format((sol_agg_SIS_orig[-1,0])*100)+ '%'
        if if_uncontained:
            sol_agg_SIS_plot = sol_agg_SIS
            str_max2 = ''
        else:
            sol_agg_SIS_plot = np.concatenate((sol_agg_SIS, sol_agg_SIS_orig), axis=1)
            str_max2 = ", {:2.2f}".format((sol_agg_SIS[-1,0])*100)+ '%'

        suptitle = '\nMaximum Ratio of Total Infectious: ' \
            + str_max1 + str_max2

        bplot(t, sol_agg_SIS_plot, plot_type=1, filesave=filesave,
              suptitle=suptitle, labels=my_labels_I, list_vl=list_t_switch,
              list_all_policies=list_all_policies, ylabel='Infectious Ratio')

        ### Aggregate SIR
        filesave_I = Path(dir_save_plots, 'SIR_I_AGG_' + str_policy+'_tf_'+str(int(t_end))+'.png')
        filesave_R = Path(dir_save_plots, 'SIR_R_AGG_' + str_policy+'_tf_'+str(int(t_end))+'.png')
        str_max_I_1 = "{:2.2f}".format(np.max(sol_agg_SIR_I_orig)*100)+ '%'
        str_max_R_1 = "{:2.2f}".format(np.max(sol_agg_SIR_R_orig)*100)+ '%'
        # Aggregate SIR I
        if if_uncontained:
            sol_agg_SIR_I_plot = sol_agg_SIR_I
            str_max2 = ''
        else:
            sol_agg_SIR_I_plot = np.concatenate((sol_agg_SIR_I_orig,
                                                 sol_agg_SIR_I,), axis=1)
            str_max2 = ", {:2.2f}".format(np.max(sol_agg_SIR_I)*100)+ '%'

        if if_uncontained:
            t_max_I = t[np.argmax(sol_agg_SIR_I_orig)]

            str_print_I = str_max_I_1
            str_print_R = str_max_R_1
        else:
            t_max_I = t[np.argmax(sol_agg_SIR_I)]
            str_print_I = ", {:2.2f}".format(np.max(sol_agg_SIR_I)*100)+ '%'
            str_print_R = ", {:2.2f}".format(np.max(sol_agg_SIR_R)*100)+ '%'

        print('x0= ', x0_vec)
        print('Max instantaneous Infectious ---> ', str_print_I[1:])
        print('Max Removed Population       ---> ', str_print_R[1:])
        print('TIME to reach max Infectious ---> ', t_max_I, ' days')


        if if_title:
            suptitle = '\nPeak/Maximum of Infectious Ratio in the population: ' + \
                str_max_I_1 + str_max2
        else:
            suptitle = ''
        bplot(t, sol_agg_SIR_I_plot, plot_type=1, filesave=filesave_I,
              suptitle=suptitle, labels=my_labels_I, list_vl=list_t_switch,
              list_all_policies=list_all_policies, ylabel='Infectious Ratio',
              cmap='Dark2')

        ### Aggregate difference between SIS and SIR outputs
        suptitle = ' Difference between solutions to SIS and SIR models'
        filesave_diff = Path(dir_save_plots, 'diff_SIS_SIR_agg_' + str_policy+'_tf_'+str(int(t_end))+'.png')
        bplot(t, sol_agg_SIS_plot-sol_agg_SIR_I_plot,
              plot_type=1, filesave=filesave_diff,
              suptitle=suptitle, labels=my_labels_I, list_vl=list_t_switch,
              list_all_policies=list_all_policies, ylabel='Infectious Ratio')

        if if_uncontained:
            sol_agg_SIR_R_plot = sol_agg_SIR_R
            str_max2 = ''
        else:
            sol_agg_SIR_R_plot = np.concatenate((sol_agg_SIR_R_orig,
                                                 sol_agg_SIR_R), axis=1)
            str_max2 = ", {:2.2f}".format(np.max(sol_agg_SIR_R)*100)+ '%'
        if if_title:
            suptitle = '\nMaximum Ratio of Removed Compartment in the population: ' + \
                str_max_R_1 + str_max2
        else:
            suptitle = ''
        bplot(t, sol_agg_SIR_R_plot, plot_type=1, filesave=filesave_R,
              suptitle=suptitle, labels=my_labels_R, list_vl=list_t_switch,
              list_all_policies=list_all_policies, ylabel='Removed Ratio',
              cmap='Dark2')


    return dict_out


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Created on Fri Mar 20 13:03:03 2020.

@author: vbokharaie
"""

def str_policy_info():
    """
    Return a string with info about defined basic policies.

    Returns
    -------
    str_out : str
        info on basic policies.

    """
    # text should be manually updated based on policies defined in get_Bopt
    str_out = """
    ***************************************************************************************
    HOW EACH POLICY IS DEFINED?
    ***************************************************************************************
    ---------------------------------------------------------------------------------------
    Below you can see the policies I have defined.

    Obviously, any other polciy can be easily defined in the code,
        when you find where they are defined.

    list_scales shows the scale for contacts rates in each group as compared to uncontained
        hence the values are in [0,1] range.

    Age groups are 0-10, 10-20, ..., 70-80 and 80+
    ---------------------------------------------------------------------------------------

    # these policies are defined intuitively,
    #------------------------
    if policy == 'Uncontained':
        w_kids = 1
        w_adults = 1
        w_old =  1
        list_scales = [w_kids, w_kids, w_adults, w_adults, w_adults, w_adults, w_adults, w_old, w_old, ]
    #------------------------
    elif policy == 'Schools_closed':
        w_kids = 0.2
        w_adults = 1
        w_old =  1
        list_scales = [w_kids, w_kids, w_adults, w_adults, w_adults, w_adults, w_adults, w_old, w_old, ]
    #------------------------
    elif policy == 'Elderly_self_isolate':
        w_kids = 1.0
        w_adults = 1.0
        w_old = 0.25
        list_scales = [w_kids, w_kids, w_adults, w_adults, w_adults, w_adults, w_adults, w_old, w_old, ]
    #------------------------
    elif policy == 'Kids_Elderly_self_isolate':
        w_kids = 0.2
        w_adults = 1.0
        w_old = 0.25
        list_scales = [w_kids, w_kids, w_adults, w_adults, w_adults, w_adults, w_adults, w_old, w_old, ]
    #------------------------
    elif policy == 'Social_Distancing':
        w_kids = 0.2
        w_adults = 0.2
        w_old = 0.25
        list_scales = [w_kids, w_kids, w_adults, w_adults, w_adults, w_adults, w_adults, w_old, w_old, ]
    #------------------------
    elif policy == 'Lockdown':
        w_kids = 0.1
        w_adults = 0.1
        w_old = 0.1
        list_scales = [w_kids, w_kids, w_adults, w_adults, w_adults, w_adults, w_adults, w_old, w_old, ]
    #------------------------
    elif policy == 'R0_is_1':
        w_kids = 0.2984
        w_adults = 0.6047
        w_old =  0.1015
        list_scales = [w_kids, w_kids, w_adults, w_adults, w_adults, w_adults, w_adults, w_old, w_old, ]
    #------------------------
    elif policy == 'Schools_Offices_closed':
        w_kids = 0.2
        w_adults = 0.5
        w_old = 1
        list_scales = [w_kids, w_kids, w_adults, w_adults, w_adults, w_adults, w_adults, w_old, w_old, ]
    #------------------------
    elif policy == 'Adults_self_isolate':
        w_adults = 0.4
        list_scales = [1, 1, w_adults, w_adults, w_adults, w_adults, w_adults, 1, 1, ]
    #------------------------
    elif policy == 'Adults_Elderly_Self_isolate':
        w_kids = 1
        w_adults = 0.2
        w_old = 0.25
        list_scales = [w_kids, w_kids, w_adults, w_adults, w_adults, w_adults, w_adults, w_old, w_old, ]
    #------------------------
    elif policy == 'Lockdown_but_kids':
        w_kids = 1
        w_adults = 0.1
        w_old = 0.1
        list_scales = [w_kids, w_kids, w_adults, w_adults, w_adults, w_adults, w_adults, w_old, w_old, ]

    """
    return str_out

def get_Bopt(file_data_opt, country='Germany', policy='Uncontained', ):
    """
    Return the contact rates for the specified model (SIS/SIR), country, pre-defined policy.

    Parameters
    ----------
    file_data_opt : TYPE
        DESCRIPTION.
    country : TYPE, optional
        DESCRIPTION. The default is 'Germany'.
    policy : TYPE, optional
        DESCRIPTION. The default is 'Uncontained'.
     : TYPE
        DESCRIPTION.

    Returns
    -------
    B_opt : TYPE
        DESCRIPTION.

    """
    # load original (uncontained) Bopt obtained from optimisation performed in matlab
    # variable names in saved files is 'B_opt_' + country, example: B_opt_Germany
    from mitepid.utils import load_mat, scale_B_opt
    varname = 'B_opt_noramlised'
    B_opt_normalised = load_mat(file_data_opt, varname)
    B_opt_orig = Bopt_normalised_2_country(B_opt_normalised, get_pop_distr(country))
    # these policies are defined intuitively,
    # all suggestions to make them more accurate are welcome
    #------------------------
    if policy == 'Uncontained':
        w_kids = 1
        w_adults = 1
        w_old =  1
        list_scales = [w_kids, w_kids, w_adults, w_adults, w_adults, w_adults, w_adults, w_old, w_old, ]
    #------------------------
    elif policy == 'Schools_closed':
        w_kids = 0.2
        w_adults = 1
        w_old =  1
        list_scales = [w_kids, w_kids, w_adults, w_adults, w_adults, w_adults, w_adults, w_old, w_old, ]
    #------------------------
    elif policy == 'Elderly_self_isolate':
        w_kids = 1.0
        w_adults = 1.0
        w_old = 0.25
        list_scales = [w_kids, w_kids, w_adults, w_adults, w_adults, w_adults, w_adults, w_old, w_old, ]
    #------------------------
    elif policy == 'Kids_Elderly_self_isolate':
        w_kids = 0.2
        w_adults = 1.0
        w_old = 0.25
        list_scales = [w_kids, w_kids, w_adults, w_adults, w_adults, w_adults, w_adults, w_old, w_old, ]
    #------------------------
    elif policy == 'Social_Distancing':
        w_kids = 0.2
        w_adults = 0.2
        w_old = 0.25
        list_scales = [w_kids, w_kids, w_adults, w_adults, w_adults, w_adults, w_adults, w_old, w_old, ]
    #------------------------
    elif policy == 'Lockdown':
        w_kids = 0.1
        w_adults = 0.1
        w_old = 0.1
        list_scales = [w_kids, w_kids, w_adults, w_adults, w_adults, w_adults, w_adults, w_old, w_old, ]
    #------------------------
    elif policy == 'R0_is_1':
        w_kids = 0.2984
        w_adults = 0.6047
        w_old =  0.1015
        list_scales = [w_kids, w_kids, w_adults, w_adults, w_adults, w_adults, w_adults, w_old, w_old, ]
    #------------------------
    elif policy == 'Schools_Offices_closed':
        w_kids = 0.2
        w_adults = 0.5
        w_old = 1
        list_scales = [w_kids, w_kids, w_adults, w_adults, w_adults, w_adults, w_adults, w_old, w_old, ]
    #------------------------
    elif policy == 'Adults_self_isolate':
        w_adults = 0.4
        list_scales = [1, 1, w_adults, w_adults, w_adults, w_adults, w_adults, 1, 1, ]
    #------------------------
    elif policy == 'Adults_Elderly_Self_isolate':
        w_kids = 1
        w_adults = 0.2
        w_old = 0.25
        list_scales = [w_kids, w_kids, w_adults, w_adults, w_adults, w_adults, w_adults, w_old, w_old, ]
    #------------------------
    elif policy == 'Lockdown_but_kids':
        w_kids = 1
        w_adults = 0.1
        w_old = 0.1
        list_scales = [w_kids, w_kids, w_adults, w_adults, w_adults, w_adults, w_adults, w_old, w_old, ]
    elif not policy == 'Uncontained':
        raise('Policy was not recognized.')
    if policy == 'Uncontained':
        B_opt = B_opt_orig
    else:
        B_opt = scale_B_opt(B_opt_orig, list_scales)

    return B_opt

def get_pop_distr(country):
    """
    Return population distribution in each country.

    in age ranges of each 10 years till 80 and 80+
    obtained from www.populationpyramid.net

    Parameters
    ----------
    country : str
        name of the country.

    Returns
    -------
    list_out : list of floats
    percentage of popultaion in each age-group.
        age groups are 0-10, 10-20, ..., 70-80, 80+

    """
    dict_pop = {}
    dict_pop['China'] = [11.9, 11.6, 13.5, 15.6, 15.6, 15.0, 10.4, 4.7, 1.7]
    dict_pop['Italy'] = [8.4, 9.5, 10.1, 11.8, 15.3, 15.7, 12.3, 9.8, 7.3]
    dict_pop['Iran'] = [17.4, 13.9, 15.5, 20.0, 13.6, 9.7, 6.2, 2.7, 1.0]
    dict_pop['SouthKorea'] = [8.4, 9.5, 13.3, 14.0, 16.3, 16.4, 12.1, 6.6, 3.5]
    dict_pop['Germany'] = [9.2, 9.6, 11.2, 12.8, 12.5, 16.2, 12.4, 9.1, 6.9]
    dict_pop['Spain'] = [9.3, 10.0, 10.0, 13.2, 17.0, 14.8, 11.0, 8.5, 6.2]
    dict_pop['France'] = [11.8, 12.0, 11.4, 12.3, 12.9, 13.2, 12.0, 8.4, 6.2]
    dict_pop['UK'] = [12.0, 11.2, 12.8, 13.7, 12.9, 13.6, 10.8, 8.5, 5.1]
    dict_pop['USA'] = [12.1, 12.9, 14.0, 13.3, 12.3, 13.0, 11.5, 7.0, 3.9]

    list_out = dict_pop[country]
    return list_out

def Bopt_normalised_2_country(Bopt, list_age_distr):
    """
    Convert the general optimised B matrix to one adapted for a certain country.

    Parameters
    ----------
    Bopt : 2D numpy array
        A Ng x Ng matrix of normalised contact rates obtained from optimisation scheme.
    list_age_distr : list of floats
        population distribution for each age range in each country.

    Returns
    -------
    Bopt_country : 2D numpy array
        Matrix of contact rates

    """
    import numpy as np
    Ng = Bopt.shape[0]
    B_factors = np.zeros(Bopt.shape)
    assert len(list_age_distr) == Ng, 'Look at this: dimension mismatch!!! :/'
    for cc1 in np.arange(Ng):
        for cc2 in np.arange(Ng):
            B_factors[cc1, cc2] = list_age_distr[cc2]/list_age_distr[cc1]
    Bopt_country = np.multiply(Bopt, B_factors)
    return Bopt_country


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Mar  9 13:50:11 2020.

@author: vbokharaie
"""


def SIR(xy, t, B, ALPHA):
    """
    Simulate SIR Model.

    Parameters
    ----------
    xy : numpy array
        array of size 2*Ng for Infective (I) and Recovered (R) trajectories.
    t : numpy array
        time array of interest.
    B : numpy 2D array
        matrix of contact rates. b_{ij} describes the influence of group j on group i.
    ALPHA : numpy array
        an array of all recovery rates.

    Returns
    -------
    array_out : numpy array
        solution to the model.

    """
    import numpy as np
    Ng = ALPHA.size;
    x = xy[:Ng]
    y = xy[Ng:]
    dxdt = np.zeros(Ng);
    dydt = np.zeros(Ng);

    for cc1 in np.arange(Ng):
        # I
        Sigma_x = 0;
        for cc2 in np.arange(Ng):
            Sigma_x = Sigma_x + B[cc1, cc2]* x[cc2];

        dxdt[cc1] = (1-x[cc1]) * Sigma_x - y[cc1] * Sigma_x - ALPHA[cc1] * x[cc1];
        # R
        dydt[cc1] = ALPHA[cc1] * x[cc1];

    array_out = np.concatenate((dxdt, dydt))
    return array_out


def SIS(x, t, B, ALPHA):
    """
    Simulate SIS model.

    Parameters
    ----------
    x : numpy array
        array of size Ng for Infective (I) trajectories.
    t : numpy array
        time array of interest.
    B : numpy 2D array
        matrix of contact rates. b_{ij} describes the influence of group j on group i.
    ALPHA : numpy array
        an array of all recovery rates.

    Returns
    -------
    dxdt : numpy array
        solution to the model.

    """
    import numpy as np
    Ng = x.size;
    dxdt = np.zeros(Ng);
    for cc1 in np.arange(Ng):
        Sigma = 0;
        for cc2 in np.arange(Ng):
            Sigma = Sigma + B[cc1, cc2]* x[cc2];

        dxdt[cc1] = (1-x[cc1]) * Sigma - ALPHA[cc1] * x[cc1];
    return dxdt





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Mar  9 13:49:51 2020.

@author: vbokharaie
"""

def bplot(t,
          sol,
          plot_type=1,
          ylim=None,
          if_show=False,
          if_save=True,
          filesave='test.png',
          labels=[''],
          suptitle='',
          list_vl=[],
          list_all_policies=None,
          ylabel='',
          if_plot_in_pc=True,
          cmap='Dark2',
          Ng=None,
          policy_name_pos=0.75,):
    """
    Line plots of the solutions to the epidemilogical model.

    Parameters
    ----------
    t : numpy array, (N_time,)
        time array
    sol : ND numpy array, (N_time x N_groups)
        solution of an SIS or SIR model, or solution of any ODE.
    plot_type : int, optional
        1: concurrent plots of all age groups, 2: each in a separate subplot. The default is 1.
    ylim : list of floats, optional
        y-axis limits. The default is (0,1).
    if_show : bool, optional
        Should plots be closed or not. The default is False.
    if_save : bool, optional
        save the plots to disk or not. The default is True.
    filesave : pathlib Path, optional
        filename for plots. The default is 'test.png'.
    labels : list of str, optional
        labels for plot legends. The default is [''].
    suptitle : str, optional
        main plot title. The default is ''.
    list_vl : list float, optional
        list of swithing times between policies, marked by red vertical lines. The default is [].
    list_all_policies : list of str, optional
        list of policies to implement at each switching time. The default is [].
    ylabel : str, optional
        y-axis label. The default is ''.
    if_plot_in_pc : bool
        y axis in percents or just the ratio.
    cmap : matplotlib colormap
        cmap used in plot
    Ng : int
        number of groups in the oringal model
    policy_name_pos : float
        in [0,1] range. Where in y axis should the text for each policy be insterted.

    Returns
    -------
    None.

    """
    import matplotlib
    # Force matplotlib to not use any Xwindows backend.
    matplotlib.use('Agg')
    import matplotlib.pyplot as plt
    import matplotlib.pylab as pl
    import matplotlib as mpl
    import numpy as np

    plt.style.use('seaborn-whitegrid')
    # plt.style.use('seaborn-darkgrid')
    # plt.style.use('bmh')
    # plt.style.use('ggplot')
    # mpl.rcParams['lines.linewidth'] = 3.0
    # mpl.rcParams['font.weight'] = 'bold'
    if plot_type == 1:
        font_size = 30
    else:
        font_size = 24
    font = {'family' : 'DejaVu Sans',
                'sans-serif' : 'Tahoma',
                'weight' : 'regular',
                'size'   : font_size}
    mpl.rc('grid', color='#316931', linewidth=1, linestyle='dotted')
    mpl.rc('font', **font)
    mpl.rc('lines', lw=3,)
    mpl.rc('xtick', labelsize=font_size)
    mpl.rc('ytick', labelsize=font_size)

    if not Ng:
        if len(sol.shape)==2:
            Ng = sol.shape[1]
        else:
            Ng = 1
            sol = sol.reshape(sol.size, 1)
    if cmap == 'viridis_r':
        colors = pl.cm.viridis_r(np.linspace(0,1,Ng))
    elif cmap == 'viridis':
        colors = pl.cm.viridis_r(np.linspace(0,1,Ng))
    elif cmap == 'Dark2':
        colors = pl.cm.Dark2.colors
    elif cmap == 'Set1':
        colors = pl.cm.Set1.colors
    if if_plot_in_pc:
        if ylabel:
            ylabel = ylabel + ' (values in %)'

        y_ax_scale = 100
    else:
        y_ax_scale = 1

    if plot_type == 1:
        fig, ax = plt.subplots(1, 1, figsize=(24,18))
        ax.set_facecolor('0.95')
        fig.subplots_adjust(bottom=0.15, top=0.92, left=0.1, right = 0.85)
        for cc in np.arange(Ng):
            # my_label = 'x'+str(cc+1).zfill(2)

            if not labels==['']:
                ax.plot(t, sol[:, cc] * y_ax_scale, label=labels[cc], color = colors[cc], alpha=0.98)
                ax.legend(bbox_to_anchor=(1.2, 1.0), prop={'size': 24})
            else:
                ax.plot(t, sol[:, cc] * y_ax_scale, color = colors[cc], alpha=0.98)

            if ylim:
                ax.set_ylim(ylim)
            ax.set_xlabel('\nTime (days)')
            ax.set_xticks(np.arange(0, t[-1], step=30))
            plt.xticks(rotation=90)
            ax.set_ylabel(ylabel)
            ylim_max = ax.get_ylim()[1]
        for idx1, xc in enumerate(list_vl):
            ax.axvline(x=xc, color='r', linestyle='--', linewidth=1)
            bbox = {'fc': '0.9', 'pad': 4, 'alpha': 0.3}
            props = {'ha': 'center', 'va': 'center', 'bbox': bbox,}
            if not list_all_policies is None:
                my_text = list_all_policies[idx1]
                # make sure policy label does not cover main plot
                idx_xc_in_t = [idx for idx, x in enumerate(t) if x>xc][0]
                if sol[idx_xc_in_t, 0] < 0.9*ylim_max and sol[idx_xc_in_t, 0] < 0.7*ylim_max:
                    policy_label_loc = policy_name_pos*ylim_max
                else:
                    policy_label_loc = 0.3*ylim_max
                xc_plot = xc+10*t[-1]/500
                ax.text(xc_plot, policy_label_loc, my_text, props, rotation=90, color='k', alpha=0.7)
        fig.suptitle(suptitle, fontsize=16, fontweight='bold')
    elif plot_type == 2:
        fig, ax_all = plt.subplots(Ng, 1, figsize=(18,18))
        fig.tight_layout()
        fig.subplots_adjust(bottom=0.1, top=0.95, left = 0.1)
        # colors = pl.cm.viridis_r(np.linspace(0,1,2))
        # fig.subplots_adjust(bottom=0.15, top=0.92, left=0.1, right = 0.88)
        for cc in np.arange(Ng):
            # my_label = 'x'+str(cc+1).zfill(2)
            ax = ax_all[cc]
            ax.plot(t, sol[:, cc] * y_ax_scale, color = colors[1], alpha=0.98)  # policy
            ax.plot(t, sol[:, cc + Ng] * y_ax_scale, color = colors[0], alpha=0.98)  # uncontained
            if ylim:
                ax.set_ylim(ylim)
            if cc == Ng-1:
                ax.set_xlabel('\nTime (days)')
                plt.xticks(rotation=90)
            else:
                ax.xaxis.set_ticklabels([])
            ax.set_xticks(np.arange(0, t[-1], step=30))
            if not labels==['']:
                ax.set_ylabel(labels[cc], rotation=90)
            ylim_max = ax.get_ylim()[1]
            for idx1, xc in enumerate(list_vl):
                ax.axvline(x=xc, color='r', linestyle='--', linewidth=1)

    if not if_show:
        plt.close('all')
    if if_save:
        dir_save = filesave.parent
        dir_save.mkdir(exist_ok=True, parents=True)
        fig.savefig(filesave, dpi=100)
        filesave_pdf = filesave.with_suffix(".pdf")
        fig.savefig(filesave_pdf, dpi=300, filetype='pdf', quality=100)



import io
import os
import re

from setuptools import find_packages
from setuptools import setup


def read(filename):
    filename = os.path.join(os.path.dirname(__file__), filename)
    text_type = type(u"")
    with io.open(filename, mode="r", encoding="utf-8") as fd:
        return re.sub(text_type(r":[a-z]+:`~?(.*?)`"), text_type(r"``\1``"), fd.read())


setup(
    name="MiTepid",
    version="0.0.2",
    url="https://github.com/vahid-sb/MiTepid_sim",
    license="GNU Version 3",
    author="Vahid Samadi Bokharaie",
    author_email="vahid.bokharaie@tuebingen.mpg.de",
    description="MiTepid_sim: Simulating a stratified model for the spread of COVID19 in any population with known age structure, Made in Tübingen. ",
    long_description=read("README.rst"),
    packages=find_packages(exclude=("tests", "venv")),
    test_suite="nose.collector",
    tests_require=["nose"],
    package_data={"mitepid": ["Optimised_B/*.*"]},
    include_package_data=True,
    install_requires=[
    "numpy",
    "scipy",
    "matplotlib",
    "pathlib",
    ],

    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
    ],
)
